# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Session',
            fields=[
                ('session_key', models.CharField(max_length=40, serialize=False, verbose_name='session key', primary_key=True)),
                ('session_data', models.TextField(verbose_name='session data')),
                ('expire_date', models.DateTimeField(verbose_name='expiry date', db_index=True)),
                ('user_agent', models.CharField(max_length=200)),
                ('last_activity', models.DateTimeField(auto_now=True)),
                ('ip', models.GenericIPAddressField()),
                ('user', models.ForeignKey(to=settings.AUTH_USER_MODEL, null=True, on_delete=models.CASCADE)),
            ],
            options={
                'verbose_name': 'session',
                'verbose_name_plural': 'sessions',
            },
            bases=(models.Model,),
        ),
    ]



import re
import warnings

from django import template
from django.contrib.gis.geoip2 import HAS_GEOIP2
from django.utils.translation import ugettext_lazy as _

register = template.Library()

BROWSERS = (
    (re.compile('Chrome'), _('Chrome')),
    (re.compile('Safari'), _('Safari')),
    (re.compile('Firefox'), _('Firefox')),
    (re.compile('Opera'), _('Opera')),
    (re.compile('IE'), _('Internet Explorer')),
)
DEVICES = (
    (re.compile('Android'), _('Android')),
    (re.compile('Linux'), _('Linux')),
    (re.compile('iPhone'), _('iPhone')),
    (re.compile('iPad'), _('iPad')),
    (re.compile('Mac OS X 10[._]9'), _('OS X Mavericks')),
    (re.compile('Mac OS X 10[._]10'), _('OS X Yosemite')),
    (re.compile('Mac OS X 10[._]11'), _('OS X El Capitan')),
    (re.compile('Mac OS X 10[._]12'), _('macOS Sierra')),
    (re.compile('Mac OS X 10[._]13'), _('macOS High Sierra')),
    (re.compile('Mac OS X'), _('OS X')),
    (re.compile('NT 5.1'), _('Windows XP')),
    (re.compile('NT 6.0'), _('Windows Vista')),
    (re.compile('NT 6.1'), _('Windows 7')),
    (re.compile('NT 6.2'), _('Windows 8')),
    (re.compile('NT 6.3'), _('Windows 8.1')),
    (re.compile('NT 10.0'), _('Windows 10')),
    (re.compile('Windows'), _('Windows')),
)


@register.filter
def device(value):
    """
    Transform a User Agent into human readable text.

    Example output:

    * Safari on iPhone
    * Chrome on Windows 8.1
    * Safari on OS X
    * Firefox
    * Linux
    * None
    """

    browser = None
    for regex, name in BROWSERS:
        if regex.search(value):
            browser = name
            break

    device = None
    for regex, name in DEVICES:
        if regex.search(value):
            device = name
            break

    if browser and device:
        return _('%(browser)s on %(device)s') % {
            'browser': browser,
            'device': device
        }

    if browser:
        return browser

    if device:
        return device

    return None


@register.filter
def location(value):
    """
    Transform an IP address into an approximate location.

    Example output:

    * Zwolle, The Netherlands
    * The Netherlands
    * None
    """
    try:
        location = geoip() and geoip().city(value)
    except Exception:
        try:
            location = geoip() and geoip().country(value)
        except Exception as e:
            warnings.warn(str(e))
            location = None
    if location and location['country_name']:
        if 'city' in location and location['city']:
            return '{}, {}'.format(location['city'], location['country_name'])
        return location['country_name']
    return None


_geoip = None


def geoip():
    global _geoip
    if _geoip is None:
        if HAS_GEOIP2:
            from django.contrib.gis.geoip2 import GeoIP2
            try:
                _geoip = GeoIP2()
            except Exception as e:
                warnings.warn(str(e))
    return _geoip



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2015-12-08 15:36
from __future__ import unicode_literals

from django.db import migrations, models
import user_sessions.models


class Migration(migrations.Migration):

    dependencies = [
        ('user_sessions', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelManagers(
            name='session',
            managers=[
                ('objects', user_sessions.models.SessionManager()),
            ],
        ),
        migrations.AlterField(
            model_name='session',
            name='ip',
            field=models.GenericIPAddressField(verbose_name='IP'),
        ),
    ]



from django.conf import settings
from django.contrib.auth import authenticate, login
from django.http import HttpRequest
from django.test import Client as BaseClient

from ..backends.db import SessionStore


class Client(BaseClient):
    """
    Custom implementation of django.test.Client.

    It is required to perform tests that require to login in sites using
    django-user-sessions since its implementation of SessionStore has to
    required parameters which is not in concordance with what is expected
    from the original Client
    """
    def login(self, **credentials):
        """
        Sets the Factory to appear as if it has successfully logged into a site.

        Returns True if login is possible; False if the provided credentials
        are incorrect, or the user is inactive, or if the sessions framework is
        not available.
        """
        user = authenticate(**credentials)
        if user and user.is_active:
            # Create a fake request to store login details.
            request = HttpRequest()
            request.user = None
            if self.session:
                request.session = self.session
            else:
                request.session = SessionStore(user_agent='Python/2.7', ip='127.0.0.1')
            login(request, user)

            # Save the session values.
            request.session.save()

            # Set the cookie to represent the session.
            session_cookie = settings.SESSION_COOKIE_NAME
            self.cookies[session_cookie] = request.session.session_key
            cookie_data = {
                'max-age': None,
                'path': '/',
                'domain': settings.SESSION_COOKIE_DOMAIN,
                'secure': settings.SESSION_COOKIE_SECURE or None,
                'expires': None,
            }
            self.cookies[session_cookie].update(cookie_data)

            return True
        else:
            return False

    def logout(self):
        """
        Removes the authenticated user's cookies and session object.

        Causes the authenticated user to be logged out.
        """
        session_cookie = self.cookies.get(settings.SESSION_COOKIE_NAME)
        if session_cookie:
            if self.session:
                self.session.delete(session_cookie)
            del self.cookies[settings.SESSION_COOKIE_NAME]

    def _session(self):
        """
        Obtains the current session variables.
        """
        if 'user_sessions' in settings.INSTALLED_APPS:
            cookie = self.cookies.get(settings.SESSION_COOKIE_NAME, None)
            if cookie:
                return SessionStore(session_key=cookie.value,
                                    user_agent='Python/2.7', ip='127.0.0.1')
    session = property(_session)



# -*- coding: UTF-8 -*-
import importlib
import logging

from django.contrib.auth import get_user_model
from django.core.management.base import BaseCommand
from user_sessions.models import Session as UserSession

logger = logging.getLogger(__name__)


def get_model_class(full_model_name):
    try:
        old_model_package, old_model_class_name = full_model_name.rsplit('.', 1)
        package = importlib.import_module(old_model_package)
        return getattr(package, old_model_class_name)
    except RuntimeError as e:
        if 'INSTALLED_APPS' in e.message:
            raise RuntimeError(
                "To run this command, temporarily append '{model}' to settings.INSTALLED_APPS"
                .format(model=old_model_package.rsplit('.models')[0]))
        raise


class Command(BaseCommand):
    """
    Convert existing (old) sessions to the user_sessions SessionStore.

    If you have an operational site and switch to user_sessions, you might want to keep your
    active users logged in. We assume the old sessions are stored in a database table `oldmodel`.
    This command creates a `user_session.Session` object for each session of the previous model.
    """
    def add_arguments(self, parser):
        parser.add_argument(
            '--oldmodel',
            dest='oldmodel',
            default='django.contrib.sessions.models.Session',
            help='Existing session model to migrate to the new UserSessions database table'
        )

    def handle(self, *args, **options):
        User = get_user_model()
        old_sessions = get_model_class(options['oldmodel']).objects.all()
        logger.info("Processing %d session objects" % old_sessions.count())
        conversion_count = 0
        for old_session in old_sessions:
            if not UserSession.objects.filter(session_key=old_session.session_key).exists():
                data = old_session.get_decoded()
                user = None
                if '_auth_user_id' in data:
                    user = User.objects.filter(pk=data['_auth_user_id']).first()
                UserSession.objects.create(
                    session_key=old_session.session_key,
                    session_data=old_session.session_data,
                    expire_date=old_session.expire_date,
                    user=user,
                    ip='127.0.0.1'
                )
                conversion_count += 1

        logger.info("Created %d new session objects" % conversion_count)



from django.conf.urls import url
from user_sessions.views import SessionDeleteOtherView

from .views import SessionDeleteView, SessionListView

app_name = 'user_sessions'
urlpatterns = [
    url(
        regex=r'^account/sessions/$',
        view=SessionListView.as_view(),
        name='session_list',
    ),
    url(
        regex=r'^account/sessions/other/delete/$',
        view=SessionDeleteOtherView.as_view(),
        name='session_delete_other',
    ),
    url(
        regex=r'^account/sessions/(?P<pk>\w+)/delete/$',
        view=SessionDeleteView.as_view(),
        name='session_delete',
    ),
]



from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _


class SessionManager(models.Manager):
    use_in_migrations = True

    def encode(self, session_dict):
        """
        Returns the given session dictionary serialized and encoded as a string.
        """
        return SessionStore().encode(session_dict)

    def save(self, session_key, session_dict, expire_date):
        s = self.model(session_key, self.encode(session_dict), expire_date)
        if session_dict:
            s.save()
        else:
            s.delete()  # Clear sessions with no data.
        return s


class Session(models.Model):
    """
    Session objects containing user session information.

    Django provides full support for anonymous sessions. The session
    framework lets you store and retrieve arbitrary data on a
    per-site-visitor basis. It stores data on the server side and
    abstracts the sending and receiving of cookies. Cookies contain a
    session ID -- not the data itself.

    Additionally this session object providers the following properties:
    ``user``, ``user_agent`` and ``ip``.
    """
    session_key = models.CharField(_('session key'), max_length=40,
                                   primary_key=True)
    session_data = models.TextField(_('session data'))
    expire_date = models.DateTimeField(_('expiry date'), db_index=True)
    objects = SessionManager()

    class Meta:
        verbose_name = _('session')
        verbose_name_plural = _('sessions')

    def get_decoded(self):
        return SessionStore(None, None).decode(self.session_data)

    user = models.ForeignKey(getattr(settings, 'AUTH_USER_MODEL', 'auth.User'),
                             null=True, on_delete=models.CASCADE)
    user_agent = models.CharField(null=True, blank=True, max_length=200)
    last_activity = models.DateTimeField(auto_now=True)
    ip = models.GenericIPAddressField(null=True, blank=True, verbose_name='IP')


# At bottom to avoid circular import
from .backends.db import SessionStore  # noqa: E402 isort:skip



from setuptools import setup, find_packages

setup(
    name='django-user-sessions',
    version='1.6.0',
    description='Django sessions with a foreign key to the user',
    long_description=open('README.rst').read(),
    author='Bouke Haarsma',
    author_email='bouke@webatoom.nl',
    url='http://github.com/Bouke/django-user-sessions',
    download_url='https://pypi.python.org/pypi/django-user-sessions',
    license='MIT',
    packages=find_packages(exclude=('example', 'tests',)),
    install_requires=['Django>=1.11'],
    include_package_data=True,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Topic :: Security',
    ],
)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.4 on 2016-12-05 15:16
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('user_sessions', '0002_auto_20151208_1536'),
    ]

    operations = [
        migrations.AlterField(
            model_name='session',
            name='ip',
            field=models.GenericIPAddressField(blank=True, null=True, verbose_name='IP'),
        ),
        migrations.AlterField(
            model_name='session',
            name='user_agent',
            field=models.CharField(blank=True, max_length=200, null=True),
        ),
    ]
